#!/bin/sh
# start tcl/tk wish -*- tcl -*- \
    exec wishx  "$0" "$@"

#  Data detectors: Parses clipboard and allows triggering appropriate commands

# This program works with "wish" and "wishx". 
# Works on Windows and Unix
# We prefer "wishx" to resolve IP/hostname in the detectors
#


# Copyright 1999 Ashish Desai and Fidelity Investments, Inc
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted, provided
# that the above copyright notice appear in all copies.  
# Ashish Desai and Fidelity Investments make no representations about the suitability 
# of this software for any purpose.  It is provided "as is" without express or
# implied warranty.  By use of this software the user agrees to 
# indemnify and hold harmless Ashish Desai and Fidelity Investments from any 
# claims or liability for loss arising out of such use.
#
# Ashish Desai, Boston
# Fidelity Investments
#
# ashish.desai@fidelity.com
# 


# Register myself with window manager to support session management
wm command . "$argv0 $argv"

# Build command to exec Microsoft windows apps.
if { $::tcl_platform(platform) == "windows" } {
    set COMSPEC [join [file split $::env(COMSPEC)] /]
} else {
    set COMSPEC {}
	set MAILER "/usr/ucb/mail"
}


#
# Detector TEMPLATE. 
# The proc must be named "somethinguseful_detector". 
#   "somethinguseful" is the text used in the checkbox to activate/deactivate detectors
# Args:
#   popup: A popup menu to which your detector should "add" entries. (Multiple entries are allowed)
#   data: The data that your detector is supposed to analyze
#
# Return value:
#   A list in the form of { {value evalCmd} {value evalCmd} } , where
#    "value" is a some unique value name to identify a menu entry, used to sort the menu entries when
#            displayed on the screen
#    "evalCmd" is some command that will be evaled to build the popup menu
#
# Please try to write code that will work on Windows and Unix. If not possible then check for "platform" and take
# appropriate action.
#

proc TEMPLATE { popup data } {
    set istrue 1
    set buttons "" ;# An array of buttons {value evalCmd}

    while { $istrue } {
	# find something in data
	set istrue [regexp -nocase -- "(https?://\[\-a-z.0-9~/%\?\=:_\]+|www\.\[\-a-z.0-9~/%\?\=:_\]+)(.*)" $data junk addr remainder ]

	# If found then build appropriate command to trigger on menu entry activation and build menu entry
	# Be frugal on the colors that you use for the menu items. Too much is bad, too little makes it 
	# hard to type of detector that got triggered
	if { $istrue == 1 } {
	     if { $::tcl_platform(platform) == "windows" } {
		 set execCmd "exec $::COMSPEC /c start ${addr} &"
	     } else {
	    set execCmd "exec netscape -remote openURL(${addr},new-window) &"
	     }
	    set cmd "$popup add command -label $addr  -command \"$execCmd\""

	    lappend buttons [list $addr $cmd]
	    set data $remainder

        }

    }
    return $buttons
}

#
# Detect http addresses
#
proc www_detector { popup data } {
    set istrue 1
    set buttons "" ;# An array of buttons {value evalCmd}

    while { $istrue } {

	set istrue [regexp -nocase -- "(https?://\[\-a-z.0-9~/%\?\=:_\]+|www\.\[\-a-z.0-9~/%\?\=:_\]+)(.*)" $data junk addr remainder ]

	if { $istrue == 1 } {
	     if { $::tcl_platform(platform) == "windows" } {
		 set execCmd "exec $::COMSPEC /c start ${addr} &"
	     } else {
	    set execCmd "exec netscape -remote openURL(${addr},new-window) &"
	     }
	    set cmd "$popup add command -label $addr  -command \"$execCmd\""

	    lappend buttons [list $addr $cmd]
	    set data $remainder

        }

    }
    return $buttons
}

#
# Detect internet email addresses
#
proc email_detector { popup data } {
    set istrue 1
    set buttons "" ;# An array of buttons {value evalCmd}
    while {$istrue} {
	
	set istrue [regexp -nocase -- "(\[\-a-z.0-9\]+@\[\-a-z.0-9\]+)(.*)" $data junk addr remainder ]

	if { $istrue == 1 } {
	     if { $::tcl_platform(platform) == "windows" } {
		 set execCmd "exec $::COMSPEC /c start mailto:${addr} &"
	     } else {
		 set execCmd "exec xterm -e $::MAILER $addr &"
	     }
	    set cmd "$popup add command -label $addr  -command \"$execCmd\""
	    lappend buttons [list $addr $cmd]
            set data $remainder
	}
    }
    return $buttons
}

#
# Detect IP addresses and does a reverse lookup on them
#
proc IP_detector { popup data } {
    set istrue 1
    set buttons "" ;# An array of buttons {value evalCmd}   
    while {$istrue} {
	
	set istrue [regexp -nocase -- "(\[0-9\]\[0-9\]*\[.\]\[0-9\].?.?\[.\]\[0-9\].?.?\[.\]\[0-9\]\[0-9\]*)(\.*)" $data junk addr remainder ]

	if { $istrue == 1 } {

	    #   use if not using wishx, just too slow for us
	    #set hostname  [exec getent hosts $addr]
	    #     split on "\t" (TAB) for answer from getent
	    #set hostname [lindex [split $hostname "	"] 1]

	    set hostname ""
	    catch {set hostname [host_info official_name $addr]}
	    set hostname "$addr: $hostname"
	    if { $::tcl_platform(platform) == "windows" } {
		set execCmd "exec $::COMSPEC /c start telnet://${addr} &"
	    } else {
		set execCmd "exec xterm -e telnet $addr &"
	    }
	    set cmd "$popup add command -label \"$hostname\"  -background lightpink -foreground black -command \"$execCmd\""
	    lappend buttons [list $addr $cmd]
            set data $remainder

        }
    }
    return $buttons
}


#
# Detect our company host addresses and do an IP lookup
#
proc hostname_detector { popup data } {
    set istrue 1
    set buttons "" ;# An array of buttons {value evalCmd}    

    while {$istrue} {
	set istrue [regexp -nocase -- " (\[a-zA-Z0-9\.-\]+.com)(\.*)" $data junk addr remainder ]
	if { $istrue == 1 } {

	    #   use if not using wishx, just too slow for us
	    #set hostname  [exec getent hosts $addr]
	    #     split on "\t" (TAB) for answer from getent
	    #set hostname [lindex [split $hostname "	"] 1]
	    


	    set hostname ""
	    catch {set hostname [host_info addresses $addr]}
	    set hostname "$addr: $hostname"
	       if { $::tcl_platform(platform) == "windows" } {
		   set execCmd "exec $::COMSPEC /c start telnet://$addr &"
	       } else {
		   set execCmd "exec xterm -e telnet $addr &"
	       }
	    set cmd "$popup add command -label \"$hostname\"  -command \"$execCmd\""
	    lappend buttons [list $addr $cmd]
            set data $remainder
	         }
    }
    return $buttons
}


#
# Detect corpid and get their names from my local database
# Corpid is of the format "a999999" where the 9 is an digit from 0-9
#
proc corpid_detector { popup data } {

    set istrue 1
    set buttons "" ;# An array of buttons {value evalCmd}

    if { [catch {set sock [socket nitro.fmrco.com 2000]}] == 1} {
	
	set cmd "$popup add command -label \"Corpdb server *DOWN*\" -background red -command exit"
	set addr "Corpdb server *DOWN*"
	lappend buttons [list $addr $cmd]

	return $buttons
    }

    while {$istrue} {
	set istrue [regexp -nocase -- "(a\[0-9\]\[0-9\]\[0-9\]\[0-9\]\[0-9\]\[0-9\])(.*)" $data junk addr remainder ]

	if { $istrue == 1 } {
	    #
	    
	    if { [info exists remember($addr) ] == 1} {  set data $remainder;continue }
	    set remember($addr) 1
	    puts $sock $addr
	    flush $sock
	    gets $sock rdata
	    if { [regexp " ACT$" $rdata] }  {set color green } else { set color yellow }
	     if { $::tcl_platform(platform) == "windows" } {
		 set execCmd "exec $::COMSPEC /c start http://mgs/whoswho/search.cgi?corp_id=$addr &"
	     } else {
		 set execCmd "exec /home/desai/bin/nr openURL(http://mgs/whoswho/search.cgi?firstname=&phone_num=&lastname=&corp_id=$addr&mailzone=&mail_id=&SEARCH=SEARCH,new-window) &"
	     }
	    set cmd "$popup add command -background $color -foreground black  -label \"$addr:   $rdata\" -command \"$execCmd\""
	    lappend buttons [list $addr $cmd]
	    
            set data $remainder
        }
    }
    puts $sock "."
    flush $sock
    close $sock
    return $buttons
}

#
# Process the selection by invoking the detectors
#
proc mainone {popup selection} {

    global toTest
    set buttonArray ""

    #self destruct popup after some delta
    after 5000 "destroy $popup"

    # Invoke detectors, the detector procs MUST be name something_detector
    foreach {detector} [info commands *_detector] {
	if { [set toTest($detector)] == 1} { 
	    set buttonArray [concat $buttonArray [$detector $popup $selection]]
	}
    }
    if {[llength $buttonArray] == 0} {return}

    # Create the buttons defined by each detector
    foreach {object} [lsort -index 0 $buttonArray] {
	set value [lindex $object 0]
	set evalCmd [lindex $object 1]
	eval $evalCmd
    }

    # Raise the popup window to where the mouse is currently located
    wm transient $popup
    wm overrideredirect $popup
    tk_popup $popup [winfo pointerx .] [winfo pointery .]
    
}

#
# Build a checkbox list of all detector we are aware off,
# Allows the user to control detector activation
#
set position 1
foreach {detector} [lsort [info commands *_detector]] {
    set detectorName [split $detector _]
    regexp "(.*)_detector" $detector junk detectorName
    #puts "$detectorName"
    set toTest(${detector}) 1
    checkbutton .b_${detectorName} -text $detectorName -variable toTest(${detector}) -onvalue 1 -offvalue 0 -width 15 -indicatoron false 
    grid .b_${detectorName} -row $position -column 0 -sticky w
    incr position
}

frame .buttons -class Detectors
grid .buttons -row $position -column 0 -stick w
button .buttons.active -text "A" 
button .buttons.askme -text "E" 
pack .buttons.active .buttons.askme -side left
bind .buttons.active <Enter> launch
bind .buttons.askme <Enter> interactiveLaunch
wm resizable . 0 0

#
# Called when we mouse over the "Activate" button.
# Can also be triggered by a "send". We do this when we want to map a hot-key in the
# window manager to a program that does the "send".
proc launch {} {
    set menuname "a[expr int(rand()*1000)]"

    if { $::tcl_platform(platform) == "windows" } {
	if {[catch {set myselection [selection get -selection CLIPBOARD]}] == 1 } {return}
    } else {
	if {[catch {set myselection [selection get -selection PRIMARY]}] == 1 } {return}
    }
    catch {destroy .$menuname}
    mainone [menu .$menuname] [list $myselection]
}

# Called when we mouse over the "Enter" button.
# Asks the user for some input, activates detectors on that input string
#
proc interactiveLaunch {} {
    set terminate_delta 8000
    catch {destroy .askme}
    toplevel .askme
    wm transient .askme
    wm overrideredirect .askme
    wm geometry .askme +[expr [winfo pointerx .] - 300 ]+[winfo pointery .]
    entry .askme.e -width 40 -textvariable s
    pack .askme.e -fill x -expand true 
    bind .askme.e <Return> { destroy .askme ; set myselection " $s" ;
	set menuname "a[expr int(rand()*1000)]"
	catch {destroy .$menuname}
	mainone [menu .$menuname] [list $myselection]
    }
    focus .askme.e
    after $terminate_delta { destroy .askme }
}


#
# To trigger the detectors through your window manager, you
# need to bind a hot-key to this script. Run the cmd below to extract the script
#
#    csplit -k -f hotkey detectors.tcl '%HOTKEY%+1' /^}/
#
proc HOTKEY {} {
#!/bin/sh
# start tcl/tk wish -*- tcl -*- \
    exec wish  "$0" "$@"

# Must use WISH as we use the "send" command to 
# communicate with the detector

# delete the top level wish window
wm withdraw .

# The process we send a message to
set detector "detectors.tcl"

if {[catch {send $detector ""}] == 1} {exit}

send $detector "launch"
exit
}
proc HOTKEY {} {}
